﻿namespace ICP.CustomRouting
{
    using System.Collections.Generic;
    using System.Web.Routing;
    
    public class CustomRoute : Route
    {
        /// <summary>
        /// it shouldn't hide the base constructors.
        /// </summary>
        public CustomRoute(string url, IRouteHandler routeHandler)
            : base(url, routeHandler)
        {
            this.DefaultUrl = url;
            this.DataTokens = new RouteValueDictionary();
        }

        /// <summary>
        /// it shouldn't hide the base constructors.
        /// </summary>
        public CustomRoute(string url, RouteValueDictionary defaults, IRouteHandler routeHandler)
            : base(url, defaults, routeHandler)
        {
            this.DefaultUrl = url;
            this.DataTokens = new RouteValueDictionary();
        }

        protected string DefaultUrl { get; set; }
        public List<string> OptionalParameters { get; set; }

        /// <summary>
        /// The routing will use this route, if it couldn't find a controller for current route 
        /// </summary>
        public Route AlternativeRoute { get; set; }

        /// <summary>
        /// Remove parameters that don't have any value
        /// </summary>
        /// <param name="requestContext"></param>
        /// <param name="values"></param>
        /// <returns></returns>
        public override VirtualPathData GetVirtualPath(RequestContext requestContext, RouteValueDictionary values)
        {
            this.Url = this.DefaultUrl;

            if (OptionalParameters != null)
            {
                foreach (var key in OptionalParameters)
                {
                    if (values[key] == null)
                    {
                        this.Url = this.Url.Replace(string.Format("/{{{0}}}", key), string.Empty);
                    }
                }
            }

            return base.GetVirtualPath(requestContext, values);
        }
    }
}
